/**
 * 建设指挥 计划进度指标面板  逻辑
 * 处理数据
 */

import { getDqtdTableData, getQqzzTableData, xxjdTableData } from "../../data/table/constructData";
import { tablePackage } from "../../dataPackage/inFrontOut";

/**前期管理 */
export async function getProphase(req, res) {
    let line = req.query.line || "全部";
    let type = req.query.type || "专项规划";

    let excelData = getQqzzTableData();

    let result:any = {};

    /**
     * "线":{
     *      yearCount:0, 
     *      data:{
     *          "单体规划建设方案":{
     *              plan:0,
     *              complete:0
     *          }
     *         
     *      } 
     * }
     */
    let dataMap = {"undefined":{
        total:0,
        data:{
            "单体规划建设方案":{plan:0, complete:0},
            "单体建设用地规划许可证":{plan:0, complete:0},
            "单体建设工程规划许可证":{plan:0, complete:0},
            "单体施工许可证":{plan:0, complete:0}
        }
    }}; 

    let conf = {
        "单体规划建设方案":[4,5], 
        "单体建设用地规划许可证":[6,7],
        "单体建设工程规划许可证":[8,9],
        "单体施工许可证":[10,11]
    };

    excelData["前期管理数据"].bodyList.forEach(subList => {
        let dataLine = subList[0];
        let dataYear = subList[1];
        if (!dataMap[dataLine]) dataMap[dataLine] = {data:{} };
        dataMap[dataLine]["total"] = subList[2];

        
        for (let name in conf) {
            let plan = subList[conf[name][0]];
            let complete = subList[conf[name][1]];
            if (!dataMap[dataLine]["data"][name]) dataMap[dataLine]["data"][name] = {plan:0, complete:0};
            dataMap[dataLine]["data"][name]["plan"] += plan;
            dataMap[dataLine]["data"][name]["complete"] += complete;
        }

    });

    result["completionStatus"] = {
        "total":0,
        "complete":0,//完成 
        "incomplete":0,//未完成
    };

    if (line == "全部") {
        let scheduleMap = {};
        let planCount = 0;
        let completeCount = 0;
        for (let key in dataMap) {
            let {data, total} = dataMap[key];
            result["completionStatus"].total += total || 0;
            for (let name in data) {
                let {plan, complete} = data[name];
                if (!scheduleMap[name]) scheduleMap[name] = {plan:0, complete:0}
                scheduleMap[name].plan += plan;
                scheduleMap[name].complete += complete;
                planCount += plan;
                completeCount += complete;
            }
        }
        result["completionStatus"].complete = completeCount;
        result["completionStatus"].incomplete = planCount-completeCount;
        let scheduleList = [];
        for (let key in scheduleMap) {
            let subComplete = scheduleMap[key].complete;
            let subPlan = scheduleMap[key].plan;

            scheduleList.push({
                name:key,
                total:subPlan,
                complete:subComplete,
                incomplete:subPlan-subComplete
            });
        }
        result["scheduleList"] = scheduleList;

    } else {
        let onceLineData = dataMap[line] || dataMap["undefined"];
        result["completionStatus"].total = onceLineData.total;
        let complete = 0;
        let scheduleList = [];
        for (let key in onceLineData.data) {
            let subComplete = onceLineData.data[key].complete;
            let subPlan = onceLineData.data[key].plan;
            complete += subComplete;
            scheduleList.push({
                name:key,
                total:subPlan,
                complete:subComplete,
                incomplete:subPlan-subComplete
            });
        }
        result["completionStatus"].complete = complete;
        result["completionStatus"].incomplete =  onceLineData.total - complete;
        result["scheduleList"] = scheduleList;
    }
    
    res.send(result);
}


/**动迁腾地 */
export async function getRelocation(req, res) {
    let line = req.query.line || "总数";
    let type = req.query.type || "专项规划";
    let year = req.query.year || "2023年";

    let excelData = getDqtdTableData();

    /**基础数据 */
    let dataMap = {};
    excelData["概览数据"].bodyList.forEach(info => {
        dataMap[info[0]] = {
            "total":info[1],
            "complete":info[2],
            "incomplete":info[3],
        }
    });

    let result:any = {};

    result["completionStatus"] = dataMap[line];


    /**详细完成情况 */
    let scheduleData = {"undefined":{
        total:0,
        data:{
            "居民":{plan:0, complete:0},
            "单位":{plan:0, complete:0},
            "绿地":{plan:0, complete:0}
        }
    }};

    let conf = {
        "居民":[2], 
        "单位":[3],
        "绿地":[4]
    };

    let index = 0;
    excelData["动迁腾地详细"].bodyList.forEach(subList => {
        let dataYear = subList[1];
        if (year == dataYear) {
            if (!scheduleData[dataYear]) scheduleData[dataYear] = {data:{} };
            scheduleData[dataYear]["total"] = subList[2];

            for (let name in conf) {
                let complete = subList[conf[name][0]];
                if (!scheduleData[dataYear]["data"][name]) scheduleData[dataYear]["data"][name] = {complete:0};
                scheduleData[dataYear]["data"][name]["complete"] += complete;
            }
            index ++;
        }
        
    });

    let scheduleMap = {};
    for (let key in scheduleData) {
        let {data} = scheduleData[key];
        for (let name in data) {
            let {complete} = data[name];
            if (!scheduleMap[name]) scheduleMap[name] = {complete:0}
            scheduleMap[name].complete += complete;
        }
    }

    let scheduleList = [];
    for (let key in scheduleMap) {
        let subComplete = scheduleMap[key].complete/index;

        scheduleList.push({
            name:key,
            complete: Math.round(subComplete)
        });
    }
    result["scheduleList"] = scheduleList;


    /**浮窗数据 */
    let detailMap = {}; 

    // let dataMap = {
    //     "undefined":{
    //         total:0,
    //         data:{
    //             "居民":{plan:0, complete:0, incomplete:0},
    //             "单位":{plan:0, complete:0, incomplete:0},
    //             "绿地":{plan:0, complete:0, incomplete:0}
    //         }
    //     }
    // }

    let detailConf = {
        "居民":[5,6],
        "单位":[7,8],
        "绿地":[9,10]
    };

    excelData["动迁腾地详细"].bodyList.forEach(subList => {
        let dataLine = subList[0];
        if (!detailMap[dataLine]) detailMap[dataLine] = {data:{} };

        for (let name in detailConf) {
            let plan = subList[detailConf[name][0]];
            let complete = subList[detailConf[name][1]];
            if (!detailMap[dataLine]["data"][name]) detailMap[dataLine]["data"][name] = {plan:0, complete:0, incomplete:0};
            detailMap[dataLine]["data"][name]["plan"] += plan;
            detailMap[dataLine]["data"][name]["complete"] += complete;
            detailMap[dataLine]["data"][name]["incomplete"] += plan - complete;
        }

    });

    result["detailList"] = detailMap;

    res.send(result);
}


/**形象进度 */
export async function getSchedule(req, res) {
    let line = req.query.line || "全部";
    let result:any = {};
    
    let excelData = xxjdTableData();
    
    let dataMap = {"undefined":{
        total:0,
        data:{
            "车站":{plan:0, complete:0, inProgress:0},
            "车辆基地":{plan:0, complete:0, inProgress:0},
            "高架区间":{plan:0, complete:0, inProgress:0}
        }
    }};

    let conf = {
        "车站":[1,2,3], 
        "车辆基地":[4,5,6],
        "高架区间":[7,8,9]
    };
    
    excelData["形象进度数据"].bodyList.forEach(subList => {
        let dataLine = subList[0];
        let dataYear = subList[1];
        if (!dataMap[dataLine]) dataMap[dataLine] = {data:{} };
        dataMap[dataLine]["total"] = subList[2];
        
        for (let name in conf) {
            let plan = subList[conf[name][0]];
            let complete = subList[conf[name][1]];
            let inProgress = subList[conf[name][2]];
            if (!dataMap[dataLine]["data"][name]) dataMap[dataLine]["data"][name] = {plan:0, complete:0, inProgress:0};
            dataMap[dataLine]["data"][name]["plan"] += plan;
            dataMap[dataLine]["data"][name]["complete"] += complete;
            dataMap[dataLine]["data"][name]["inProgress"] += inProgress;
        }

    });

    if (line == "全部") {
        let scheduleMap = {};
        let planCount = 0;
        let completeCount = 0;
        let inProgressCount = 0;
        for (let key in dataMap) {
            let {data} = dataMap[key];
            for (let name in data) {
                let {plan, complete, inProgress} = data[name];
                if (!scheduleMap[name]) scheduleMap[name] = {plan:0, complete:0, inProgress:0}
                scheduleMap[name].plan += plan;
                scheduleMap[name].complete += complete;
                scheduleMap[name].inProgress += inProgress;
                planCount += plan;
                completeCount += complete;
                inProgressCount += inProgress;
            }
        }
        let scheduleList = [];
        for (let key in scheduleMap) {
            let subComplete = scheduleMap[key].complete;
            let subPlan = scheduleMap[key].plan;
            let subInProgress = scheduleMap[key].inProgress;

            scheduleList.push({
                name:key,
                total:subPlan,
                complete:subComplete,
                incomplete:subInProgress,
                notStarted:subPlan-subComplete-subInProgress
            });
        }
        result["scheduleList"] = scheduleList;

    } else {
        let onceLineData = dataMap[line] || dataMap["undefined"];
        let complete = 0;
        let scheduleList = [];
        for (let key in onceLineData.data) {
            let subComplete = onceLineData.data[key].complete;
            let subPlan = onceLineData.data[key].plan;
            let subInProgress = onceLineData.data[key].inProgress;
            complete += subComplete;
            scheduleList.push({
                name:key,
                total:subPlan,
                complete:subComplete,
                incomplete:subInProgress,
                notStarted:subPlan-subComplete-subInProgress
            });
        }

        result["scheduleList"] = scheduleList;
    }

    let czxxDataMap = {};
    excelData["车站详细数据"].bodyList.forEach(subList => {
        let dataLine = subList[0];
        let dataYear = subList[1];
        if (!czxxDataMap[dataLine]) czxxDataMap[dataLine] = {data:{} };
        czxxDataMap[dataLine]["total"] = subList[2];

        let plan = subList[2];
        let complete = subList[3];
        if (!czxxDataMap[dataLine]["data"][dataYear]) czxxDataMap[dataLine]["data"][dataYear] = {plan:0, complete:0};
        czxxDataMap[dataLine]["data"][dataYear]["plan"] += plan;
        czxxDataMap[dataLine]["data"][dataYear]["complete"] += complete;
    })

    if(line == "全部") {
        let scheduleMap = {};
        let planCount = 0;
        let completeCount = 0;
        for (let key in czxxDataMap) {
            let {data, total} = czxxDataMap[key];
            for (let name in data) {
                let {plan, complete} = data[name];
                if (!scheduleMap[name]) scheduleMap[name] = {plan:0, complete:0}
                scheduleMap[name].plan += plan;
                scheduleMap[name].complete += complete;
                planCount += plan;
                completeCount += complete;
            }
        }

        let scheduleList = [];
        for (let key in scheduleMap) {
            let subComplete = scheduleMap[key].complete;
            let subPlan = scheduleMap[key].plan;

            scheduleList.push({
                name:key,
                total:subPlan,
                complete:subComplete
            });
        }
        result["czxxsj"] = scheduleList;
    }  else {
        let onceLineData = czxxDataMap[line] || czxxDataMap["undefined"];
        let complete = 0;
        let scheduleList = [];
        for (let key in onceLineData.data) {
            let subComplete = onceLineData.data[key].complete;
            let subPlan = onceLineData.data[key].plan;
            complete += subComplete;
            scheduleList.push({
                name:key,
                total:subPlan,
                complete:subComplete
            });
        }
        result["czxxsj"] = scheduleList;
    }

    res.send(result);
}

export function getDetailData() {

}

export async function getMilestone(req, res) {

    let result:any = {};
    result["detailedList"] = [
        {
            title:"23号线首座车站开工",
            time:"2022-09-19", 
            desc:"23号线江川东路站于2022-9-19首幅低下连续墙开工。", 
            img:"http://192.168.0.71:40001/2005_01.jpg"
        },
        {
            title:"23号线首座车站开工",
            time:"2022-09-19", 
            desc:"23号线江川东路站于2022-9-19首幅低下连续墙开工。", 
            img:"http://192.168.0.71:40001/2005_01.jpg"
        },
       
    ];

    res.send(result);
}


